@@ -9,7 +9,7 @@ def format_file(fp, indents=2, top_comment=False, align_by_equal_sign=False, fla
     """
     Format protobuf file, override the original file if no new file path(new_fp) specified
     or write the formatted result to a new file.
-    :param fp: unformatted protobuf file path.
+    :param fp: malformed protobuf file path.
     :param indents: indents number.
     :param top_comment: convert the single line comment at the left of code as a top comment or not. Default: not
     :param align_by_equal_sign: if align the code by equal sign or not.
@@ -48,8 +48,7 @@ def format_file(fp, indents=2, top_comment=False, align_by_equal_sign=False, fla
     :param new_fp: the file path of new formatted protobuf file. Rewrite the original file if it is not specified.
     :return: file content size.
     """
-    proto_parser = ProtoParser()
-    protobuf_obj = proto_parser.load(fp=fp)
+    protobuf_obj = ProtoParser.load(fp=fp)
     content = protobuf_obj.to_string(
         indents=indents,
         top_comment=top_comment,
@@ -108,11 +107,10 @@ def format_str(proto_str, indents=2, top_comment=False, align_by_equal_sign=Fals
          message Outer {
              int32 outerq = 1;
          }
-    :param comment_max_length: the max length of comment line, defalut is no limitation.
+    :param comment_max_length: the max length of comment line, default is no limitation.
     :return: formatted string.
     """
-    proto_parser = ProtoParser()
-    protobuf_obj = proto_parser.loads(proto_str=proto_str.strip())
+    protobuf_obj = ProtoParser.loads(proto_str=proto_str.strip())
     content = protobuf_obj.to_string(
         indents=indents,
         top_comment=top_comment,
