@@ -5,49 +5,41 @@ class Position(Enum):
     LEFT = 'left'
     Right = 'right'
     TOP = 'top'
-    BELOW = 'below'
+    BOTTOM = 'bottom'
 
 
-class Comment():
+class Comment:
     def __init__(self, text: str, position: Position):
         self.text = text
         self.position = position
 
 
-class Syntax():
+class Syntax:
     def __init__(self, value, comments: list):
         self.value = value
         self.comments = comments
 
 
-class Package():
+class Package:
     def __init__(self, value, comments: list):
         self.value = value
         self.comments = comments
 
 
-class Import():
+class Import:
     def __init__(self, value, comments: list):
         self.value = value
         self.comments = comments
 
 
-class Option():
+class Option:
     def __init__(self, name, value, comments: list):
         self.name = name
         self.value = value
         self.comments = comments
 
-class Element():
-    def __init__(self, type, name, number, annotation='', label='', comments=[]):
-        self.label = label
-        self.type = type
-        self.name = name
-        self.number = number
-        self.annotation = annotation
-        self.comments = comments
 
-class MessageElement():
+class Element:
     def __init__(self, type, name, number, annotation='', label='', comments=[]):
         self.label = label
         self.type = type
@@ -57,28 +49,7 @@ class MessageElement():
         self.comments = comments
 
 
-class Message():
-    def __init__(self, name, elements=[], comments=[]):
-        self.name = name
-        self.elements = elements
-        self.comments = comments
-
-class ExtendElement():
-    def __init__(self, type, name, number, annotation='', label='', comments=[]):
-        self.label = label
-        self.type = type
-        self.name = name
-        self.number = number
-        self.annotation = annotation
-        self.comments = comments
-
-class Extend():
-    def __init__(self, name, elements=[], comments=[]):
-        self.name = name
-        self.elements = elements
-        self.comments = comments
-
-class OneofElement():
+class MessageElement:
     def __init__(self, type, name, number, annotation='', label='', comments=[]):
         self.label = label
         self.type = type
@@ -88,31 +59,88 @@ class OneofElement():
         self.comments = comments
 
 
-class Oneof():
-    def __init__(self, name, elements=[], comments=[]):
+class Message:
+    def __init__(self, name, elements=None, comments=None):
+        if elements is None:
+            elements = []
+        if comments is None:
+            comments = []
         self.name = name
         self.elements = elements
         self.comments = comments
 
 
-class EnumElement():
-    def __init__(self, name, number, annotation='', comments=[]):
+class ExtendElement:
+    def __init__(self, type, name, number, annotation='', label='', comments=None):
+        if comments is None:
+            comments = []
+        self.label = label
+        self.type = type
         self.name = name
         self.number = number
         self.annotation = annotation
         self.comments = comments
 
 
-class ProtoEnum():
-    def __init__(self, name, elements=[], comments=[]):
+class Extend:
+    def __init__(self, name, elements=None, comments=None):
+        if comments is None:
+            comments = []
+        if elements is None:
+            elements = []
         self.name = name
         self.elements = elements
         self.comments = comments
 
 
-#  rpc SeatAvailability (SeatAvailabilityRequest) returns (SeatAvailabilityResponse);
-class ServiceElement():
-    def __init__(self, label, name, request, response, comments=[]):
+class OneofElement:
+    def __init__(self, type, name, number, annotation='', label='', comments=None):
+        if comments is None:
+            comments = []
+        self.label = label
+        self.type = type
+        self.name = name
+        self.number = number
+        self.annotation = annotation
+        self.comments = comments
+
+
+class Oneof:
+    def __init__(self, name, elements=None, comments=None):
+        if comments is None:
+            comments = []
+        if elements is None:
+            elements = []
+        self.name = name
+        self.elements = elements
+        self.comments = comments
+
+
+class EnumElement:
+    def __init__(self, name, number, annotation='', comments=None):
+        if comments is None:
+            comments = []
+        self.name = name
+        self.number = number
+        self.annotation = annotation
+        self.comments = comments
+
+
+class ProtoEnum:
+    def __init__(self, name, elements=None, comments=None):
+        if comments is None:
+            comments = []
+        if elements is None:
+            elements = []
+        self.name = name
+        self.elements = elements
+        self.comments = comments
+
+
+class ServiceElement:
+    def __init__(self, label, name, request, response, comments=None):
+        if comments is None:
+            comments = []
         self.label = label
         self.name = name
         self.request = request
@@ -120,8 +148,12 @@ class ServiceElement():
         self.comments = comments
 
 
-class Service():
-    def __init__(self, name, elements=[], comments=[]):
+class Service:
+    def __init__(self, name, elements=None, comments=None):
+        if comments is None:
+            comments = []
+        if elements is None:
+            elements = []
         self.name = name
         self.elements = elements
         self.comments = comments
