@@ -14,13 +14,24 @@ from .proto_structures import Oneof
 from .util import to_lines
 
 
-class Protobuf():
+class Config:
     SPACES_BETWEEN_VALUE_COMMENT = 2
     SPACES_BEFORE_AFTER_EQUAL_SIGN = 1
     SPACES_BETWEEN_NUMBER_ANNOTATION = 1
     ONE_SPACE = ' '
     TOP_COMMENT_INDENTS = ' ' * 4
 
+    def __init__(self, indents_unit=2, align_by_equal_sign=None, top_comment=False, flatten=False,
+                 comment_max_length=None):
+        self.indents_unit = indents_unit
+        self.align_by_equal_sign = align_by_equal_sign
+        self.top_comment = top_comment
+        self.flatten = flatten
+        self.comment_max_length = comment_max_length
+
+
+class Protobuf:
+
     def __init__(self):
         self.syntax = None
         self.package = None
@@ -28,35 +39,27 @@ class Protobuf():
         self.imports = []
         self.objects = []
 
-        self.indents_unit = 2
-        self.align_by_equal_sign = None
-        self.top_comment = False
-        self.flatten = False
-        self.comment_max_length = None
+        self.config = Config()
 
     def to_string(self, indents=2, align_by_equal_sign=None, top_comment=False, flatten=False, comment_max_length=None):
-        if flatten:
+        self.config = Config(indents_unit=indents, align_by_equal_sign=align_by_equal_sign, top_comment=top_comment,
+                             flatten=flatten, comment_max_length=comment_max_length)
+        if self.config.flatten:
             self.flatten_objects()
 
-        self.indents_unit = indents
-        self.align_by_equal_sign = align_by_equal_sign
-        self.top_comment = top_comment
-        self.flatten = flatten
-        self.comment_max_length = comment_max_length
-
         syntax_string = self.syntax_string()
         package_string = self.package_string()
         option_string = self.options_string()
         imports_string = self.imports_string()
 
         all_lines = []
-        for object in self.objects:
+        for obj in self.objects:
             lines = []
-            self.format_object_without_comment(object, lines, indents=0)
+            self.format_object_without_comment(obj, lines, indents=0)
             max_length = self.get_max_length(lines)
-            extra = self.get_max_lengthes(lines)
+            extra = self.get_max_lengths(lines)
             new_lines = []
-            self.format_object(object, new_lines, 0, extra)
+            self.format_object(obj, new_lines, 0, extra)
             all_lines.append('\n'.join(new_lines))
 
         object_string = '\n\n'.join(all_lines)
@@ -95,15 +98,16 @@ class Protobuf():
 
         new_objects.append(new_obj)
 
-    def get_max_length(self, lines):
-        max = 0
+    @staticmethod
+    def get_max_length(lines):
+        length = 0
         for line in lines:
-            if max < len(line):
-                max = len(line)
+            if length < len(line):
+                length = len(line)
 
-        return max
+        return length
 
-    def get_max_lengthes(self, lines):
+    def get_max_lengths(self, lines):
         max_equal_sign_index = 0
         max_length_of_number = 0
         max_length_of_object_line = 0
@@ -130,7 +134,7 @@ class Protobuf():
                 if max_length_of_object_line < len(line):
                     max_length_of_object_line = len(line)
 
-        max_length_sample = f'{s1.rstrip()}{self.ONE_SPACE * self.SPACES_BEFORE_AFTER_EQUAL_SIGN}={self.ONE_SPACE * self.SPACES_BEFORE_AFTER_EQUAL_SIGN}{s2};'
+        max_length_sample = f'{s1.rstrip()}{self.config.ONE_SPACE * self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN}={self.config.ONE_SPACE * self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN}{s2};'
         max_length = max(len(max_length_sample), max_length_of_service_element_line, max_length_of_object_line)
 
         return {
@@ -146,7 +150,7 @@ class Protobuf():
 
         line = f'syntax = "{self.syntax.value}";'
 
-        return self.make_string(line, 0, self.syntax.comments, self.SPACES_BETWEEN_VALUE_COMMENT)
+        return self.make_string(line, 0, self.syntax.comments, self.config.SPACES_BETWEEN_VALUE_COMMENT)
 
     def package_string(self):
         if not self.package:
@@ -154,7 +158,7 @@ class Protobuf():
 
         line = f'package {self.package.value};'
 
-        return self.make_string(line, 0, self.package.comments, self.SPACES_BETWEEN_VALUE_COMMENT)
+        return self.make_string(line, 0, self.package.comments, self.config.SPACES_BETWEEN_VALUE_COMMENT)
 
     def options_string(self):
         if not self.options:
@@ -188,7 +192,7 @@ class Protobuf():
         else:
             line = f'option {option.name} = "{option.value}";'
 
-        space_between_number_comment = max_length - len(line) + self.SPACES_BETWEEN_VALUE_COMMENT
+        space_between_number_comment = max_length - len(line) + self.config.SPACES_BETWEEN_VALUE_COMMENT
 
         return self.make_string(line, 0, option.comments, space_between_number_comment)
 
@@ -217,7 +221,7 @@ class Protobuf():
 
     def import_string(self, obj: Import, max_length):
         line = f'import "{obj.value}";'
-        space_between_number_comment = max_length - len(line) + self.SPACES_BETWEEN_VALUE_COMMENT
+        space_between_number_comment = max_length - len(line) + self.config.SPACES_BETWEEN_VALUE_COMMENT
 
         return self.make_string(line, 0, obj.comments, space_between_number_comment)
 
@@ -240,12 +244,17 @@ class Protobuf():
 
         space_between_number_comment = 2
         if max_length:
-            space_between_number_comment = max_length - len(line) + self.SPACES_BETWEEN_VALUE_COMMENT
+            space_between_number_comment = max_length - len(line) + self.config.SPACES_BETWEEN_VALUE_COMMENT
 
         if no_comment:
             return self.make_indented_line(line, indents=indents)
         else:
-            return self.make_string(line, indents, obj.comments, space_between_number_comment)
+            comments = [comment for comment in obj.comments if comment.position is not Position.BOTTOM]
+            return self.make_string(line, indents, comments, space_between_number_comment)
+
+    def create_object_rear(self, obj, indents):
+        bottom_comments = [comment for comment in obj.comments if comment.position == Position.BOTTOM]
+        return self.make_string('}', indents, bottom_comments, self.config.SPACES_BETWEEN_VALUE_COMMENT)
 
     def get_object_element_class(self, obj):
         classes = {
@@ -282,56 +291,56 @@ class Protobuf():
             if isinstance(element, element_class):
                 line = element_str_method(
                     element,
-                    indents + self.indents_unit,
+                    indents + self.config.indents_unit,
                     True,
-                    self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
-                    self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
-                    self.SPACES_BETWEEN_VALUE_COMMENT
+                    self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                    self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                    self.config.SPACES_BETWEEN_VALUE_COMMENT
                 )
 
-                if self.align_by_equal_sign:
+                if self.config.align_by_equal_sign:
                     line_without_number = line.split('=')[0].rstrip()
                     space_between_name_equal_sign = max_equal_sign_index - len(line_without_number)
 
                     if hasattr(element, 'number'):
                         space_between_number_comment = max_length - max_equal_sign_index - len('=') - len(';') - len(
                             element.number) - len(
-                            element.annotation) - self.SPACES_BEFORE_AFTER_EQUAL_SIGN - self.SPACES_BETWEEN_NUMBER_ANNOTATION + self.SPACES_BETWEEN_VALUE_COMMENT
+                            element.annotation) - self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN - self.config.SPACES_BETWEEN_NUMBER_ANNOTATION + self.config.SPACES_BETWEEN_VALUE_COMMENT
 
                         if element.annotation == '':
                             # the space between number and rules is tripped, so shouldn't count it
-                            space_between_number_comment = space_between_number_comment + self.SPACES_BETWEEN_NUMBER_ANNOTATION
+                            space_between_number_comment = space_between_number_comment + self.config.SPACES_BETWEEN_NUMBER_ANNOTATION
 
                     elif line.strip().startswith('rpc'):
-                        space_between_number_comment = max_length - len(line) + self.SPACES_BETWEEN_VALUE_COMMENT
+                        space_between_number_comment = max_length - len(line) + self.config.SPACES_BETWEEN_VALUE_COMMENT
                     else:
                         space_between_number_comment = max_length - len(
-                            line) - self.SPACES_BEFORE_AFTER_EQUAL_SIGN + self.SPACES_BETWEEN_VALUE_COMMENT
+                            line) - self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN + self.config.SPACES_BETWEEN_VALUE_COMMENT
 
                     string = element_str_method(
                         element,
-                        indents + self.indents_unit,
+                        indents + self.config.indents_unit,
                         False,
                         space_between_name_equal_sign,
-                        self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                        self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
                         space_between_number_comment
                     )
                 else:
-                    space_between_number_comment = max_length - len(line) + self.SPACES_BETWEEN_VALUE_COMMENT
+                    space_between_number_comment = max_length - len(line) + self.config.SPACES_BETWEEN_VALUE_COMMENT
                     string = element_str_method(
                         element,
-                        indents + self.indents_unit,
+                        indents + self.config.indents_unit,
                         False,
-                        self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
-                        self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                        self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                        self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
                         space_between_number_comment
                     )
 
                 string_list.append(string)
             else:
-                self.format_object(element, string_list, indents + self.indents_unit, extra)
+                self.format_object(element, string_list, indents + self.config.indents_unit, extra)
 
-        message_rear = self.make_string('}', indents, [], self.SPACES_BETWEEN_VALUE_COMMENT)
+        message_rear = self.create_object_rear(obj, indents)
         string_list.append(message_rear)
 
     def format_object_without_comment(self, obj, string_list, indents):
@@ -344,15 +353,15 @@ class Protobuf():
             if isinstance(element, element_class):
                 string = element_str_method(
                     element,
-                    indents + self.indents_unit,
+                    indents + self.config.indents_unit,
                     True,
-                    self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
-                    self.SPACES_BEFORE_AFTER_EQUAL_SIGN,
-                    self.SPACES_BETWEEN_VALUE_COMMENT
+                    self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                    self.config.SPACES_BEFORE_AFTER_EQUAL_SIGN,
+                    self.config.SPACES_BETWEEN_VALUE_COMMENT
                 )
                 string_list.append(string)
             else:
-                self.format_object_without_comment(element, string_list, indents=indents + self.indents_unit)
+                self.format_object_without_comment(element, string_list, indents=indents + self.config.indents_unit)
 
         message_rear = self.make_indented_line('}', indents=indents)
         string_list.append(message_rear)
@@ -366,7 +375,7 @@ class Protobuf():
             space_between_equal_sign_number,
             space_between_number_comment
     ):
-        line = f'{obj.label} {obj.type} {obj.name}{self.ONE_SPACE * space_between_name_equal_sign}={self.ONE_SPACE * space_between_equal_sign_number}{obj.number}{self.ONE_SPACE * self.SPACES_BETWEEN_NUMBER_ANNOTATION}{obj.annotation}'
+        line = f'{obj.label} {obj.type} {obj.name}{self.config.ONE_SPACE * space_between_name_equal_sign}={self.config.ONE_SPACE * space_between_equal_sign_number}{obj.number}{self.config.ONE_SPACE * self.config.SPACES_BETWEEN_NUMBER_ANNOTATION}{obj.annotation}'
         line = line.strip()  # remove prefix space when obj.label is empty string or rules is empty.
         line = f'{line};'
 
@@ -401,7 +410,7 @@ class Protobuf():
             space_between_equal_sign_number,
             space_between_number_comment
     ):
-        line = f'{obj.name}{self.ONE_SPACE * space_between_name_equal_sign}={self.ONE_SPACE * space_between_equal_sign_number}{obj.number}{self.ONE_SPACE * self.SPACES_BETWEEN_NUMBER_ANNOTATION}{obj.annotation}'
+        line = f'{obj.name}{self.config.ONE_SPACE * space_between_name_equal_sign}={self.config.ONE_SPACE * space_between_equal_sign_number}{obj.number}{self.config.ONE_SPACE * self.config.SPACES_BETWEEN_NUMBER_ANNOTATION}{obj.annotation}'
         line = line.strip()  # remove prefix space when rules is empty.
         line = f'{line};'
 
@@ -430,28 +439,23 @@ class Protobuf():
     def make_string(self, value_line, indents, comments, space_between_number_comment):
         top_comment_lines = []
         right_comment = ''
-        if self.top_comment:
+        bottom_comment_lines = []
+        if self.config.top_comment:
             for comment in comments:
                 if comment.position == Position.TOP:
-                    text_lines = [l.strip() for l in comment.text.split('\n')]
-
-                    new_text_lines = []
-                    if self.comment_max_length is not None:
-                        for l in text_lines:
-                            new_text_lines.extend(to_lines(l, self.comment_max_length))
-                        text_lines = new_text_lines
-
-                    top_comment_lines.extend(text_lines)
+                    top_comment_lines.extend(self.make_top_comments(comment))
+                if comment.position == Position.BOTTOM:
+                    bottom_comment_lines.extend(self.make_top_comments(comment))
                 if comment.position == Position.Right:
                     line = comment.text
                     text_lines = [line]
 
                     new_text_lines = []
-                    if self.comment_max_length is not None:
+                    if self.config.comment_max_length is not None:
                         for l in text_lines:
-                            new_text_lines.extend(to_lines(l, self.comment_max_length))
+                            new_text_lines.extend(to_lines(l, self.config.comment_max_length))
 
-                    if self.comment_max_length is not None:
+                    if self.config.comment_max_length is not None:
                         top_comment_lines.extend(new_text_lines)
                     else:
                         top_comment_lines.extend(text_lines)
@@ -460,35 +464,49 @@ class Protobuf():
         else:
             for comment in comments:
                 if comment.position == Position.TOP:
-                    text_lines = [l.strip() for l in comment.text.split('\n')]
-
-                    new_text_lines = []
-                    if self.comment_max_length is not None:
-                        for l in text_lines:
-                            new_text_lines.extend(to_lines(l, self.comment_max_length))
-                        text_lines = new_text_lines
-
-                    top_comment_lines.extend(text_lines)
+                    top_comment_lines.extend(self.make_top_comments(comment))
+                if comment.position == Position.BOTTOM:
+                    bottom_comment_lines.extend(self.make_top_comments(comment))
                 if comment.position == Position.Right:
                     right_comment = comment.text
 
         indented_top_comment_lines = []
         if top_comment_lines:
             for comment_line in top_comment_lines:
-                indented_top_comment_lines.append(f'**{self.TOP_COMMENT_INDENTS}{comment_line}')
+                indented_top_comment_lines.append(f'**{self.config.TOP_COMMENT_INDENTS}{comment_line}')
             indented_top_comment_lines.insert(0, '/*')
             indented_top_comment_lines.append('*/')
 
+        indented_bottom_comment_lines = []
+        if bottom_comment_lines:
+            for comment_line in bottom_comment_lines:
+                indented_bottom_comment_lines.append(f'**{self.config.TOP_COMMENT_INDENTS}{comment_line}')
+            indented_bottom_comment_lines.insert(0, '/*')
+            indented_bottom_comment_lines.append('*/')
+
         lines = indented_top_comment_lines
         if right_comment:
-            line = f'{value_line}{self.ONE_SPACE * space_between_number_comment}// {right_comment}'
+            line = f'{value_line}{self.config.ONE_SPACE * space_between_number_comment}// {right_comment}'
             lines.append(line)
         else:
             lines.append(value_line)
 
-        indented_lines = [f'{self.ONE_SPACE * indents}{line}' for line in lines]
+        if indented_bottom_comment_lines:
+            lines.extend(indented_bottom_comment_lines)
+
+        indented_lines = [f'{self.config.ONE_SPACE * indents}{line}' for line in lines]
         string = '\n'.join(indented_lines)
         return string
 
     def make_indented_line(self, line, indents=0):
-        return f'{self.ONE_SPACE * indents}{line}'
+        return f'{self.config.ONE_SPACE * indents}{line}'
+
+    def make_top_comments(self, comment):
+        text_lines = [line.strip() for line in comment.text.split('\n')]
+
+        new_text_lines = []
+        if self.config.comment_max_length is not None:
+            for line in text_lines:
+                new_text_lines.extend(to_lines(line, self.config.comment_max_length))
+            text_lines = new_text_lines
+        return text_lines
